==> tunesbrowser-0.2.0.tar.bz2: tunesbrowser-0.2.0/misc_ui.c <==
/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "lists.h"
#include "tunesbrowser.h"

static GtkLabel *playstatus;
static GtkLabel *play_str;

static GtkLabel *playpause_label;
static GtkImage *playpause_image;

enum songstatus currentStatus = GUISTATUS_STOPPED;

static void cb_update_songstatus(enum songstatus status, daap_host *host, int songid);
static void cb_update_songpos(int seconds);

typedef struct
{
    enum songstatus s;
    daap_host *host;
    int songid;
} guistatus;

/*********** async command callbacks ***********/

static void async_cmd_cb_sourcelist_draw(__UNUSED__ void *cb_data)
{
    sourcelist_redraw();
}

static void async_cmd_cb_artistlist_draw(__UNUSED__ void *cb_data)
{
    artistlist_redraw();
}

static void async_cmd_cb_albumlist_draw(__UNUSED__ void *cb_data)
{
    albumlist_redraw();
}

static void async_cmd_cb_songlist_draw(__UNUSED__ void *cb_data)
{
    songlist_redraw();
}

static void async_cmd_cb_set_songpos(void *cb_data)
{
    int seconds = (int)cb_data;
    cb_update_songpos(seconds);
}

static void async_cmd_cb_set_status(void *cb_data)
{
    guistatus *status = (guistatus*)cb_data;
    cb_update_songstatus(status->s, status->host,
                         status->songid);

    free(status);
}

/****************** insert message ************/
void set_songstatus(enum songstatus status, daap_host *host, int songid)
{
    guistatus *gstatus = malloc(sizeof(guistatus));
    gstatus->s = status;
    gstatus->host = host;
    gstatus->songid = songid;
    async_cmd_queue_add(async_cmd_cb_set_status, (void*)gstatus);
}

void set_songpos(int seconds)
{
    async_cmd_queue_add(async_cmd_cb_set_songpos, (void*)seconds);
}

void schedule_lists_draw(int source, int artist, int album, int song)
{
    if (source) async_cmd_queue_add(async_cmd_cb_sourcelist_draw, NULL);
    if (artist) async_cmd_queue_add(async_cmd_cb_artistlist_draw, NULL);
    if (album) async_cmd_queue_add(async_cmd_cb_albumlist_draw, NULL);
    if (song) async_cmd_queue_add(async_cmd_cb_songlist_draw, NULL);
}


/****************** async callback handlers *************/

static void cb_update_songstatus(enum songstatus status, daap_host *host, int songid)
{
    currentStatus = status;

    switch(status)
    {
        case GUISTATUS_STOPPED:
            gtk_label_set_text(play_str, "Not Playing");
            gtk_label_set_text(playstatus, "-");

            songlist_set_playing(NULL, 0);
            partyshuffle_set_playing(NULL, 0);
            /* fall through */

        case GUISTATUS_PAUSED:
            gtk_label_set_text(playpause_label, "Play");
            gtk_image_set_from_stock(playpause_image, "gtk-yes", 4);

            if (status == GUISTATUS_STOPPED)
                break;

            /* fall through */
        case GUISTATUS_PLAYING:
            {
                DAAP_ClientHost_DatabaseItem song;
                char *str;

                if (host)
                {
                    daap_host_get_song_item(host, songid, &song);
                    if (song.songartist)
                    {
                        str = malloc(strlen(song.songartist) + strlen(" - ") +
                                     strlen(song.itemname) + 1);
                        strcpy(str, song.songartist);
                        strcat(str, " - ");
                    }
                    else
                    {
                        str = malloc(strlen(song.itemname) + 1);
                        str[0] = 0;
                    }
                    strcat(str, song.itemname);
                }
                else
                {
                    str = malloc(strlen("File Source")+1);
                    strcpy(str, "File Source");
                }

                gtk_label_set_text(play_str, str);

                free(str);

                switch (daap_get_playsource())
                {
                case PLAYSOURCE_HOST:
                    partyshuffle_set_playing(NULL, 0);
                    songlist_set_playing(host, songid);
                    break;
                case PLAYSOURCE_PARTY:
                    songlist_set_playing(NULL, 0);
                    partyshuffle_set_playing(host, songid);
                    break;
                default:
                    break;
                }
            }
            if (status == GUISTATUS_PAUSED)
                break;
            gtk_label_set_text(playpause_label, "Pause");
            gtk_image_set_from_stock(playpause_image, "gtk-no", 4);

            break;
    }
}

static void cb_update_songpos(int seconds)
{
    if (currentStatus == GUISTATUS_STOPPED)
    {
        return;
    }

    if (seconds >= 0)
    {
        int minutes = seconds / 60;
        seconds -= minutes * 60;
        char bufstr[11] = {0};

        snprintf(bufstr, 10, "%i:%02i",
                 minutes, seconds);

        gtk_label_set_text(playstatus, bufstr);
    }
    else
    {
        gtk_label_set_text(playstatus, "-");
    }
}

/********** standard callbacks *******/
void on_playpause_clicked(__UNUSED__ GtkButton *playpause, __UNUSED__ gpointer user_data)
{
    daap_host *host = daap_host_get_visible();
    if (!host) return;

    if (currentStatus == GUISTATUS_PLAYING) /* current play -> pause */
    {
        /* FIXME shouldn't really call audioplayer from here */
        set_songstatus(GUISTATUS_PAUSED, host, songlist_get_selected_id());
        audioplayer_pause();
    }
    else if (currentStatus == GUISTATUS_STOPPED) /* current stopped -> play */
    {

        if (partyshuffle_is_active())
        {
            partyshuffle_play_selected();
        }
        else
        {
            /* FIXME songlist should have a play_selected function */
            int id = songlist_get_selected_id();
            if (id == -1) return;
            daap_host_play_song(PLAYSOURCE_HOST, host, id);
        }
    }
    else if (currentStatus == GUISTATUS_PAUSED) /* current paused -> resume */
    {
        /* FIXME shouldn't call audioplayer from here */
        set_songstatus(GUISTATUS_PLAYING, host, songlist_get_selected_id());
        audioplayer_resume();
    }
}

/************** init **************/

void misc_ui_init(GladeXML *xml)
{
    GtkButton *playpause;

    playstatus = GTK_LABEL(glade_xml_get_widget(xml, "playstatus"));
    play_str = GTK_LABEL(glade_xml_get_widget(xml, "play_str"));

    playpause_label = GTK_LABEL(glade_xml_get_widget(xml, "playpause_label"));
    playpause_image = GTK_IMAGE(glade_xml_get_widget(xml, "playpause_image"));

    playpause = GTK_BUTTON(glade_xml_get_widget(xml, "playpause"));
    g_signal_connect(G_OBJECT(playpause), "clicked",
                     G_CALLBACK(on_playpause_clicked),
                     NULL);
}

void misc_ui_finalize()
{
}

==> tunesbrowser-0.3.0.tar.bz2: tunesbrowser-0.3.0/misc_ui.c <==
/* Copyright (c) 2004 David Hammerton - All rights reserved
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */


#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "debug.h"
#include "lists.h"
#include "tunesbrowser.h"

#define DEFAULT_DEBUG_CHANNEL "misc_ui"

GtkWindow *tunesbrowser_win;

static GtkLabel *playstatus;
static GtkLabel *play_str;

static GtkLabel *playpause_label;
static GtkImage *playpause_image;

static GtkWidget *browser;

enum songstatus currentStatus = GUISTATUS_STOPPED;

static void cb_update_songstatus(enum songstatus status, daap_host *host, int songid);
static void cb_update_songpos(int seconds);

typedef struct
{
    enum songstatus s;
    daap_host *host;
    int songid;
} guistatus;

/*********** async command callbacks ***********/

static void async_cmd_cb_sourcelist_draw(__UNUSED__ void *cb_data)
{
    TRACE("(...)\n");
    sourcelist_redraw();
}

static void async_cmd_cb_artistlist_draw(__UNUSED__ void *cb_data)
{
    TRACE("(...)\n");
    artistlist_redraw();
}

static void async_cmd_cb_albumlist_draw(__UNUSED__ void *cb_data)
{
    TRACE("(...)\n");
    albumlist_redraw();
}

static void async_cmd_cb_songlist_draw(__UNUSED__ void *cb_data)
{
    TRACE("(...)\n");
    songlist_redraw();
}

static void async_cmd_cb_browser_visible(void *cb_data)
{
    int visible = (int)cb_data;

    if (visible)
    {
        gtk_widget_show_all(browser);
    }
    else
    {
        gtk_widget_hide_all(browser);
    }
}

static void async_cmd_cb_set_songpos(void *cb_data)
{
    int seconds = (int)cb_data;
    TRACE("(%i)\n", seconds);
    cb_update_songpos(seconds);
}

static void async_cmd_cb_set_status(void *cb_data)
{
    guistatus *status = (guistatus*)cb_data;
    TRACE("(%i)\n", status);
    cb_update_songstatus(status->s, status->host,
                         status->songid);

    free(status);
}

/****************** insert message ************/
void set_songstatus(enum songstatus status, daap_host *host, int songid)
{
    guistatus *gstatus = malloc(sizeof(guistatus));
    gstatus->s = status;
    gstatus->host = host;
    gstatus->songid = songid;
    async_cmd_queue_add(async_cmd_cb_set_status, (void*)gstatus);
}

void set_songpos(int seconds)
{
    async_cmd_queue_add(async_cmd_cb_set_songpos, (void*)seconds);
}

void schedule_lists_draw(int source, int artist, int album, int song)
{
    if (source) async_cmd_queue_add(async_cmd_cb_sourcelist_draw, NULL);
    if (artist) async_cmd_queue_add(async_cmd_cb_artistlist_draw, NULL);
    if (album) async_cmd_queue_add(async_cmd_cb_albumlist_draw, NULL);
    if (song) async_cmd_queue_add(async_cmd_cb_songlist_draw, NULL);
}

void schedule_browser_visible(int visible)
{
    async_cmd_queue_add(async_cmd_cb_browser_visible, (void*)visible);
}

/****************** async callback handlers *************/

static void cb_update_songstatus(enum songstatus status, daap_host *host, int songid)
{
    currentStatus = status;

    switch(status)
    {
        case GUISTATUS_STOPPED:
            gtk_label_set_text(play_str, "Not Playing");
            gtk_label_set_text(playstatus, "-");

            songlist_set_playing(NULL, 0);
            partyshuffle_set_playing(NULL, 0);
            /* fall through */

        case GUISTATUS_PAUSED:
            gtk_label_set_text(playpause_label, "Play");
            gtk_image_set_from_stock(playpause_image, "gtk-yes", 4);

            if (status == GUISTATUS_STOPPED)
                break;

            /* fall through */
        case GUISTATUS_PLAYING:
            {
                DAAP_ClientHost_DatabaseItem song;
                char *str;

                if (host)
                {
                    daap_host_get_song_item(host, songid, &song);
                    if (song.songartist)
                    {
                        str = malloc(strlen(song.songartist) + strlen(" - ") +
                                     strlen(song.itemname) + 1);
                        strcpy(str, song.songartist);
                        strcat(str, " - ");
                    }
                    else
                    {
                        str = malloc(strlen(song.itemname) + 1);
                        str[0] = 0;
                    }
                    strcat(str, song.itemname);
                }
                else
                {
                    str = malloc(strlen("File Source")+1);
                    strcpy(str, "File Source");
                }

                gtk_label_set_text(play_str, str);

                free(str);

                switch (daap_get_playsource())
                {
                case PLAYSOURCE_HOST:
                    partyshuffle_set_playing(NULL, 0);
                    songlist_set_playing(host, songid);
                    break;
                case PLAYSOURCE_PARTY:
                    songlist_set_playing(NULL, 0);
                    partyshuffle_set_playing(host, songid);
                    break;
                default:
                    break;
                }
            }
            if (status == GUISTATUS_PAUSED)
                break;
            gtk_label_set_text(playpause_label, "Pause");
            gtk_image_set_from_stock(playpause_image, "gtk-no", 4);

            break;
    }
}

static void cb_update_songpos(int seconds)
{
    if (currentStatus == GUISTATUS_STOPPED)
    {
        return;
    }

    if (seconds >= 0)
    {
        int minutes = seconds / 60;
        seconds -= minutes * 60;
        char bufstr[11] = {0};

        snprintf(bufstr, 10, "%i:%02i",
                 minutes, seconds);

        gtk_label_set_text(playstatus, bufstr);
    }
    else
    {
        gtk_label_set_text(playstatus, "-");
    }
}

/********** standard callbacks *******/
void on_playpause_clicked(__UNUSED__ GtkButton *playpause, __UNUSED__ gpointer user_data)
{
    daap_host *host;

    TRACE("(...)\n");

    host = daap_host_get_visible();
    if (!host) return;

    if (currentStatus == GUISTATUS_PLAYING) /* current play -> pause */
    {
        int song_id = -1;
        TRACE("pausing\n");
        /* FIXME shouldn't really call audioplayer from here */
        daap_host_get_playing_info(&host, &song_id);
        if (host == NULL || song_id == -1)
        {
            ERR("inconsistance state. expect playing on pause\n");
            return;
        }
        set_songstatus(GUISTATUS_PAUSED, host, song_id);
        audioplayer_pause();
    }
    else if (currentStatus == GUISTATUS_STOPPED) /* current stopped -> play */
    {
        TRACE("playing\n");

        if (partyshuffle_is_active())
        {
            partyshuffle_play_selected();
        }
        else
        {
            /* FIXME songlist should have a play_selected function */
            int id = songlist_get_selected_id();
            if (id == -1) return;
            daap_host_play_song(PLAYSOURCE_HOST, host, id);
        }
    }
    else if (currentStatus == GUISTATUS_PAUSED) /* current paused -> resume */
    {
        int song_id = -1;
        TRACE("resuming\n");
        /* FIXME shouldn't call audioplayer from here */
        daap_host_get_playing_info(&host, &song_id);
        if (host == NULL || song_id == -1)
        {
            ERR("inconsistance state. expect playing on pause\n");
            return;
        }
        set_songstatus(GUISTATUS_PLAYING, host, song_id);
        audioplayer_resume();
    }
}

/************** init **************/

static gboolean startup_dialog_idle_init(__UNUSED__ gpointer data)
{
    /* don't show donate, so show tip if we can */
    if (!pref_get_showdonate())
    {
        if (pref_get_showtips()) tips_show();
    }
    /* don't show tip, but we can show donate */
    else if (!pref_get_showtips())
    {
        donate_show();
    }
    /* we can show either */
    else
    {
        /* if the tip hasn't been shown, do that */
        if (!pref_get_tipshownonce())
        {
            tips_show();
        }
        /* otherwise randomly pick one */
        else if (g_random_boolean())
        {
            donate_show();
        }
        else
        {
            tips_show();
        }
    }
    return FALSE;
}

static void on_donate_menu_activate(__UNUSED__ GtkMenuItem *menuitem,
                                    __UNUSED__ gpointer user_data)
{
    donate_show();
}

static void on_tip_menu_activate(__UNUSED__ GtkMenuItem *menuitem,
                                 __UNUSED__ gpointer user_data)
{
    tips_show();
}

static void on_quit_menu_activate(__UNUSED__ GtkMenuItem *menuitem,
                                  __UNUSED__ gpointer user_data)
{
    gtk_main_quit();
}

static void menu_init(GladeXML *xml)
{
    GtkMenuItem *donate_menu;
    GtkMenuItem *tip_menu;
    GtkMenuItem *quit_menu;

    donate_menu = GTK_MENU_ITEM(glade_xml_get_widget(xml, "donate_menu"));
    tip_menu = GTK_MENU_ITEM(glade_xml_get_widget(xml, "tip_menu"));
    quit_menu = GTK_MENU_ITEM(glade_xml_get_widget(xml, "quit_menu"));

    g_signal_connect(G_OBJECT(donate_menu), "activate",
                     G_CALLBACK(on_donate_menu_activate), NULL);
    g_signal_connect(G_OBJECT(tip_menu), "activate",
                     G_CALLBACK(on_tip_menu_activate), NULL);
    g_signal_connect(G_OBJECT(quit_menu), "activate",
                     G_CALLBACK(on_quit_menu_activate), NULL);
}

void misc_ui_init(GladeXML *xml)
{
    GtkButton *playpause;

    tunesbrowser_win = GTK_WINDOW(glade_xml_get_widget(xml, "tunesbrowser_win"));

    browser = GTK_WIDGET(glade_xml_get_widget(xml, "browser"));

    playstatus = GTK_LABEL(glade_xml_get_widget(xml, "playstatus"));
    play_str = GTK_LABEL(glade_xml_get_widget(xml, "play_str"));

    playpause_label = GTK_LABEL(glade_xml_get_widget(xml, "playpause_label"));
    playpause_image = GTK_IMAGE(glade_xml_get_widget(xml, "playpause_image"));

    playpause = GTK_BUTTON(glade_xml_get_widget(xml, "playpause"));
    g_signal_connect(G_OBJECT(playpause), "clicked",
                     G_CALLBACK(on_playpause_clicked),
                     NULL);

    menu_init(xml);

    g_idle_add(startup_dialog_idle_init, NULL);
}

void misc_ui_finalize()
{
}
